@@ -277,3 +277,66 @@
 #'   ison_lawfirm
 #'   ```
 "ison_lawfirm"
+
+# Star Wars ####
+
+#' Seven one-mode Star Wars character interactions (Gabasova 2016)
+#'
+#' @description
+#' One-mode network dataset collected by Gabasova (2016)
+#' on the interactions between Star Wars characters in each movie from
+#' Episode 1 (The Phantom Menace) to Episode 7 (The Force Awakens).
+#' There is a separate network for each episode,
+#' and the data is listed in order from episode 1 to 7.
+#' The network for each episode varies in the number of nodes and ties.
+#' For all networks, characters are named (eg. R2-D2, Anakin, Chewbacca)
+#' and the following node attributes are provided where available:
+#' height, mass, hair color, skin color, eye color, birth year, sex, homeworld, and species.
+#' Weighted ties represent the number of times characters speak 
+#' within the same scene of the film.
+#' @details
+#' The network for each episode may be extracted and used separately,
+#' eg. `ison_starwars[[1]]` or `ison_starwars$Episode I` for Episode 1.
+#' @docType data
+#' @keywords datasets
+#' @name ison_starwars
+#' @usage data(ison_starwars)
+#' @references
+#'   Gabasova, E. (2016).
+#'   \emph{Star Wars social network.}.
+#'   \doi{10.5281/zenodo.1411479}
+#' @format
+#'   ```{r, echo = FALSE}
+#'   ison_starwars
+#'   ```
+"ison_starwars"
+
+# Friends ####
+
+#' One-mode Friends character connections (McNulty, 2020)
+#'
+#' @description
+#' One-mode network collected by \href{https://github.com/keithmcnulty/friends_analysis/}{McNulty (2020)}
+#' on the connections between the Friends TV series characters
+#' from Seasons 1 to 10.
+#' The `ison_friends` is a directed network
+#' containing connections between characters organised by season number,
+#' which is reflected in the tie attribute 'season'.
+#' The network contains 650 nodes
+#' Each tie represents the connection between a character pair (appear in the same scene),
+#' and the weight of the tie is the number of scenes the character pair appears in together.
+#' For all networks, characters are named (eg. Phoebe, Ross, Rachel).
+#' @details
+#' The data contains both networks but each may be used separately.
+#' @docType data
+#' @keywords datasets
+#' @name ison_friends
+#' @usage data(ison_friends)
+#' @references
+#'   McNulty, K. (2020).
+#'   \emph{Network analysis of Friends scripts.}.
+#' @format
+#'   ```{r, echo = FALSE}
+#'   ison_friends
+#'   ```
+"ison_friends"
